@@ -1,25 +1,31 @@
 /*
- * Copyright (c) 2015, 2016, Oracle and/or its affiliates. All rights reserved.
- *
- * This code is licensed under the terms of the GPLv2
- * <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
- * MySQL Connectors. There are special exceptions to the terms and
- * conditions of the GPLv2 as it is applied to this software, see the
- * FLOSS License Exception
- * <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published
- * by the Free Software Foundation; version 2 of the License.
+ * it under the terms of the GNU General Public License, version 2.0, as
+ * published by the Free Software Foundation.
+ *
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation. The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
+ *
+ * Without limiting anything contained in the foregoing, this file,
+ * which is part of Connector/C++, is also subject to the
+ * Universal FOSS Exception, version 1.0, a copy of which can be found at
+ * https://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- * for more details.
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License, version 2.0, for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
@@ -36,9 +42,6 @@
 #include <map>
 #include <sstream>
 
-PUSH_BOOST_WARNINGS
-#include <boost/lexical_cast.hpp>
-POP_BOOST_WARNINGS
 
 using ::std::cout;
 using ::std::endl;
@@ -704,9 +707,8 @@ void Session_crud::drop_table(cdk::Session &sess, const Table_ref &tbl)
 
 void Session_crud::drop_coll(cdk::Session &sess, const Table_ref &coll)
 {
-  Any_list& args = const_cast<Any_list&>(static_cast<const Any_list&>(coll));
-
-  Reply drop(sess.admin("drop_collection", args));
+  Reply drop(sess.admin("drop_collection",
+                        static_cast<const cdk::Any::Document&>(coll)));
   drop.wait();
 
   if (0 < drop.entry_count()
@@ -720,10 +722,8 @@ void Session_crud::drop_coll(cdk::Session &sess, const Table_ref &coll)
 void Session_crud::create_coll(cdk::Session &sess, const Table_ref &coll)
 {
   drop_coll(sess, coll);
-
-  Any_list& args = const_cast<Any_list&>(static_cast<const Any_list&>(coll));
-
-  Reply create(sess.admin("create_collection", args));
+  Reply create(sess.admin("create_collection",
+                          static_cast<const cdk::Any::Document&>(coll)));
   create.wait();
 
   if (0 < create.entry_count()
@@ -810,7 +810,6 @@ private:
 
 // =====================================================================
 
-
 CRUD_TEST_BEGIN(find)
 {
   Session sess(this);
@@ -1097,6 +1096,59 @@ CRUD_TEST_BEGIN(update)
     // TODO: Add data checks
   }
 
+  cout << endl
+    << "Do a merge and change the document layout" << endl
+    << " - rename name to name2" << endl
+    << " - flatten date {day: 23, month: \"April\"} to" << endl
+    << "   day2: 23, month2: \"April\""
+    << endl << endl;
+
+  {
+    struct Update : public Update_spec
+    {
+      Update() : Update_spec(2)
+      {}
+
+      void process(Update_processor &prc) const
+      {
+          Safe_prc<Expression::Document::Processor> sprc(prc.patch()->doc());
+          Path name_path("name");
+          Path day_path("date", "day");
+          Path mon_path("date", "month");
+
+          sprc->doc_begin();
+          // rename "name" to "name2"
+          sprc->key_val("name2")->scalar()->ref(name_path);
+          sprc->key_val("name")->scalar()->val()->null();
+
+          // Flatten "date: { day : 23, month : apr }" into
+          // "day2 : 23, month2: apr"
+          sprc->key_val("date")->scalar()->val()->null();
+          sprc->key_val("day2")->scalar()->ref(day_path);
+          sprc->key_val("month2")->scalar()->ref(mon_path);
+          sprc->doc_end();
+      }
+    }
+    update_spec;
+
+    static Expr which_update("name = 'baz'");
+
+    Reply update(sess.coll_update(coll, &which_update, update_spec));
+    update.wait();
+
+    // Show data after update.
+    static Expr which_find("name2 = 'baz' AND day2 = 23");
+    Reply find(sess.coll_find(coll, NULL, &which_find));
+    Cursor c(find);
+    set_meta_data(c);
+    c.get_rows(*this);
+    c.wait();
+    EXPECT_EQ(1, rows.size());
+
+    // TODO: Add data checks
+  }
+
+
   // TODO: array update operations
   // TODO: Update rows in a table
 
@@ -1827,3 +1879,36 @@ CRUD_TEST_BEGIN(views)
   cout <<"Done!" <<endl;
 }
 CRUD_TEST_END
+
+
+CRUD_TEST_BEGIN(upsert)
+{
+  Session sess(this);
+  SKIP_IF_SERVER_VERSION_LESS(sess, 8, 0, 3)
+
+  Doc_list doc_list;
+  doc_list.add(1, "coo", 10);
+  doc_list.add(2, "roo", 20);
+  doc_list.add(3, "moo", 30);
+
+  Reply r = sess.coll_add(coll, doc_list, &doc_list.params());
+  r.wait();
+
+  Doc_list upsert_list;
+  upsert_list.add(1, "zoo", 40);
+  r = sess.coll_add(coll, upsert_list,
+                   &upsert_list.params(), true);
+  r.wait();
+  EXPECT_EQ(0, r.entry_count());
+
+  Doc_list no_upsert_list;
+  no_upsert_list.add(1, "noo", 50);
+  r = sess.coll_add(coll, no_upsert_list,
+                    &no_upsert_list.params(), false);
+  r.wait();
+  EXPECT_EQ(1, r.entry_count());
+
+  cout << "Done!" << endl;
+
+}
+CRUD_TEST_END
