@@ -1,25 +1,31 @@
 /*
- * Copyright (c) 2015, 2016, Oracle and/or its affiliates. All rights reserved.
- *
- * This code is licensed under the terms of the GPLv2
- * <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
- * MySQL Connectors. There are special exceptions to the terms and
- * conditions of the GPLv2 as it is applied to this software, see the
- * FLOSS License Exception
- * <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published
- * by the Free Software Foundation; version 2 of the License.
+ * it under the terms of the GNU General Public License, version 2.0, as
+ * published by the Free Software Foundation.
+ *
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation. The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
+ *
+ * Without limiting anything contained in the foregoing, this file,
+ * which is part of Connector/C++, is also subject to the
+ * Universal FOSS Exception, version 1.0, a copy of which can be found at
+ * https://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- * for more details.
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License, version 2.0, for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef CDK_COMMON_H
@@ -29,14 +35,11 @@
 #include "api/obj_ref.h"
 #include "api/expression.h"
 #include "api/document.h"
-#include "protocol/mysqlx.h"
+#include "api/query.h"
+
 
 namespace cdk {
 
-using protocol::mysqlx::row_count_t;
-using protocol::mysqlx::col_count_t;
-using protocol::mysqlx::charset_id_t;
-
 
 /*
   CDK value type system
@@ -141,6 +144,14 @@ protected:
   ==========
 */
 
+/*
+  Note: These types are compatible with the X protocol.
+*/
+
+typedef uint64_t row_count_t;
+typedef uint32_t col_count_t;
+typedef uint64_t collation_id_t;
+
 
 class Column_info
     : public api::Column_ref
@@ -151,6 +162,7 @@ public:
 
   virtual length_t length() const =0;
   virtual length_t decimals() const =0;
+  virtual collation_id_t collation() const = 0;
 };
 
 
@@ -159,6 +171,7 @@ struct Traits
   typedef cdk::row_count_t row_count_t;
   typedef cdk::col_count_t col_count_t;
   typedef void transaction_id_t;
+  typedef const string& savepoint_id_t;
 
   typedef cdk::Type_info   Type_info;
   typedef const cdk::Format_info& Format_info;
@@ -166,6 +179,7 @@ struct Traits
 };
 
 
+
 typedef api::Meta_data<Traits> Meta_data;
 
 
@@ -284,7 +298,7 @@ class Expr_processor;
 typedef api::Any<Expr_processor>    Expression;
 typedef api::Expr_list<Expression>  Expr_list;
 
-using cdk::protocol::mysqlx::api::Doc_path;
+using api::Doc_path;
 
 class Expr_processor
 {
@@ -314,8 +328,11 @@ public:
   virtual Args_prc*   op(const char*) =0;
   virtual Args_prc*   call(const Object_ref&) =0;
 
+  // TODO: consider changing ref() so that they return doc path processor
+
   virtual void ref(const Column_ref&, const Doc_path*) =0;
   virtual void ref(const Doc_path&) =0;
+
   virtual void param(const string&) =0;
   virtual void param(uint16_t) =0;
   virtual void var(const string&) =0;
@@ -397,7 +414,7 @@ public:
   virtual void num(double)        =0;
   virtual void yesno(bool)        =0;
 
-  virtual ~JSON_processor() {}
+  virtual ~JSON_processor() NOEXCEPT {}
 };
 
 typedef api::Doc_base<JSON_processor> JSON;
@@ -502,6 +519,25 @@ class Doc_source
 {};
 
 
+/*
+  Classes for describing statement parameters
+  ===========================================
+*/
+
+typedef cdk::api::Limit<row_count_t>         Limit;
+typedef cdk::api::Order_by<Expression>       Order_by;
+typedef cdk::api::Sort_direction             Sort_direction;
+typedef cdk::api::Doc_base<Value_processor>  Param_source;
+typedef cdk::api::Lock_mode::value           Lock_mode_value;
+typedef cdk::api::Lock_contention::value     Lock_contention_value;
+
+using   cdk::api::View_security;
+using   cdk::api::View_algorithm;
+using   cdk::api::View_check;
+
+typedef cdk::api::View_options               View_options;
+typedef cdk::api::View_spec<View_options>    View_spec;
+
 /*
   Classes for describing update operations
   ========================================
@@ -578,6 +614,11 @@ public:
 
   virtual Expr_prc* array_append(const Doc_path*) =0;
 
+  /*
+    Perform MERGE_PATCH operation on a document.
+  */
+  virtual Expr_prc* patch() =0;
+
 };
 
 
@@ -629,6 +670,9 @@ struct Safe_prc<Update_processor>
 
   Safe_prc<Expr_prc> array_append(const Doc_path *path)
   { return m_prc ? m_prc->array_append(path) : NULL; }
+
+  Safe_prc<Expr_prc> patch()
+  { return m_prc ? m_prc->patch() : NULL; }
 };
 
 }  // cdk
