@@ -1,22 +1,22 @@
 /*
- * Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
  * published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
- * documentation.  The authors of MySQL hereby grant you an
- * additional permission to link the program and your derivative works
- * with the separately licensed software that they have included with
- * MySQL.
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation. The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * Without limiting anything contained in the foregoing, this file,
- * which is part of MySQL Connector/C++, is also subject to the
+ * which is part of Connector/C++, is also subject to the
  * Universal FOSS Exception, version 1.0, a copy of which can be found at
- * http://oss.oracle.com/licenses/universal-foss-exception.
+ * https://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,9 +25,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
-
 #ifndef CDK_FOUNDATION_COMMON_H
 #define CDK_FOUNDATION_COMMON_H
 
@@ -43,7 +42,7 @@
 #define DIAGNOSTIC_PUSH_CDK  PRAGMA_CDK(warning (push))
 #define DIAGNOSTIC_POP_CDK   PRAGMA_CDK(warning (pop))
 
-#elif defined __GNUC__ || defined __clang__ 
+#elif defined __GNUC__ || defined __clang__
 
 #define PRAGMA_CDK(X) _Pragma(#X)
 #define DISABLE_WARNING_CDK(W) PRAGMA_CDK(GCC diagnostic ignored #W)
@@ -114,6 +113,25 @@
 #define POP_SYS_WARNINGS_CDK  DIAGNOSTIC_POP_CDK
 
 
+/*
+  Macros to disable false positives from -Warray-bounds checks for lines
+  of code that were manually verified to be correct.
+*/
+
+#ifdef _MSC_VER
+
+#define PUSH_ARRAY_BOUNDS_WARNING_CDK DIAGNOSTIC_PUSH_CDK
+#define POP_ARRAY_BOUNDS_WARNING_CDK DIAGNOSTIC_POP_CDK
+
+#else
+
+#define PUSH_ARRAY_BOUNDS_WARNING_CDK \
+  DIAGNOSTIC_PUSH_CDK DISABLE_WARNING_CDK(-Warray-bounds)
+#define POP_ARRAY_BOUNDS_WARNING_CDK DIAGNOSTIC_POP_CDK
+
+#endif
+
+
 // Avoid warnings from Protobuf includes
 
 #if defined _MSC_VER
