@@ -1,22 +1,22 @@
 /*
- * Copyright (c) 2015, 2020, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
  * published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
- * documentation.  The authors of MySQL hereby grant you an
- * additional permission to link the program and your derivative works
- * with the separately licensed software that they have included with
- * MySQL.
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation. The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * Without limiting anything contained in the foregoing, this file,
- * which is part of MySQL Connector/C++, is also subject to the
+ * which is part of Connector/C++, is also subject to the
  * Universal FOSS Exception, version 1.0, a copy of which can be found at
- * http://oss.oracle.com/licenses/universal-foss-exception.
+ * https://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +25,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef CDK_MYSQLX_RESULT_H
@@ -94,7 +94,7 @@ cdk::Charset::value get_collation_cs(collation_id_t id)
   */
 
   if (0 == id)
-    return cdk::Charset::utf8;
+    return cdk::Charset::utf8mb3;
 
 #undef  CS
 #define COLL_TO_CS(CS) COLLATIONS_##CS(COLL_TO_CS_CASE) return cdk::Charset::CS;
@@ -446,12 +446,21 @@ public:
     assert(m_session);
   }
 
-  virtual ~Stmt_op()
+  virtual ~Stmt_op() NOEXCEPT
   {
-    discard();
-    wait();
-    if (m_session)
-      m_session->deregister_stmt(this);
+    try
+    {
+      discard();
+      wait();
+    }
+    catch (...) {}
+
+    try
+    {
+      if (m_session)
+        m_session->deregister_stmt(this);
+    }
+    catch (...) {}
   }
 
   Session& get_session()
@@ -812,7 +821,7 @@ protected:
 public:
 
   Cursor(const std::shared_ptr<Stmt_op> &reply);
-  ~Cursor();
+  ~Cursor() NOEXCEPT;
 
   void get_rows(mysqlx::Row_processor& rp);
   void get_rows(mysqlx::Row_processor& rp, row_count_t limit);
