@@ -1,25 +1,31 @@
 /*
- * Copyright (c) 2015, 2016, Oracle and/or its affiliates. All rights reserved.
- *
- * This code is licensed under the terms of the GPLv2
- * <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
- * MySQL Connectors. There are special exceptions to the terms and
- * conditions of the GPLv2 as it is applied to this software, see the
- * FLOSS License Exception
- * <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published
- * by the Free Software Foundation; version 2 of the License.
+ * it under the terms of the GNU General Public License, version 2.0, as
+ * published by the Free Software Foundation.
+ *
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation. The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
+ *
+ * Without limiting anything contained in the foregoing, this file,
+ * which is part of Connector/C++, is also subject to the
+ * Universal FOSS Exception, version 1.0, a copy of which can be found at
+ * https://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- * for more details.
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License, version 2.0, for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 /*
@@ -190,6 +196,8 @@ class Find
 
 public:
 
+  Lock_mode_value m_lock_mode = Lock_mode_value::NONE;
+
   Find(const Db_obj &obj,
     const Expression *criteria, row_count_t limit, row_count_t skip = 0)
     : m_obj(obj)
@@ -243,6 +251,10 @@ private:
     return NULL;
   }
 
+  const Lock_mode_value locking() const
+  {
+    return m_lock_mode;
+  }
 };
 
 
@@ -880,5 +892,66 @@ TEST_F(Protocol_mysqlx_xplugin, crud_projections)
 
 }
 
+TEST_F(Protocol_mysqlx_xplugin, row_locking)
+{
+  SKIP_IF_NO_XPLUGIN;
+
+  authenticate();
+  SKIP_IF_SERVER_VERSION_LESS(8, 0, 3)
+
+  do_query(L"DROP DATABASE IF EXISTS crud_test_db");
+  do_query(L"CREATE DATABASE crud_test_db");
+  do_query(L"USE crud_test_db");
+  do_query(L"DROP TABLE IF EXISTS row_locking");
+  do_query(L"CREATE TABLE row_locking(id int primary key)");
+  do_query(L"INSERT INTO row_locking(id) VALUES (1),(2),(3)");
+
+  do_query(L"BEGIN");
+
+  Protocol &proto = get_proto();
+
+  Db_obj db_obj("row_locking", "crud_test_db");
+  Find   find1(db_obj, NULL, 10, 0);
+  find1.m_lock_mode = Lock_mode_value::EXCLUSIVE;
+
+  cout << "Find" << endl;
+  proto.snd_Find(TABLE, find1).wait();
+
+  cout << "Metadata" << endl;
+  Mdata_handler mdh;
+  proto.rcv_MetaData(mdh).wait();
+
+  cout << "Fetch rows" << endl;
+  Row_handler_crud rhc;
+
+  rhc.row_ids.push_back(1);
+  rhc.row_ids.push_back(2);
+  rhc.row_ids.push_back(3);
+
+  proto.rcv_Rows(rhc).wait();
+
+  Stmt_handler sh;
+  proto.rcv_StmtReply(sh).wait(); // Expect OK
+
+  proto.snd_StmtExecute("sql",
+    L"select IF(trx_rows_locked > 0, 1, 0) rows_locked from information_schema.innodb_trx " \
+    "where trx_mysql_thread_id = connection_id()", NULL).wait();
+
+  cout << "Metadata for locked rows number" << endl;
+  proto.rcv_MetaData(mdh).wait();
+
+  cout << "Fetch number of locked rows" << endl;
+
+  // The number of rows locked will be the number of rows in the table + 1
+  rhc.row_ids.clear();
+  rhc.row_ids.push_back(1);
+  proto.rcv_Rows(rhc).wait();
+  proto.rcv_StmtReply(sh).wait(); // Expect OK
+
+  do_query(L"COMMIT");
+  do_query(L"DROP TABLE IF EXISTS row_locking");
+}
+
+
 }}}  // cdk::test::proto
 
