@@ -1,22 +1,22 @@
 /*
- * Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
  * published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
- * documentation.  The authors of MySQL hereby grant you an
- * additional permission to link the program and your derivative works
- * with the separately licensed software that they have included with
- * MySQL.
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation. The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * Without limiting anything contained in the foregoing, this file,
- * which is part of MySQL Connector/C++, is also subject to the
+ * which is part of Connector/C++, is also subject to the
  * Universal FOSS Exception, version 1.0, a copy of which can be found at
- * http://oss.oracle.com/licenses/universal-foss-exception.
+ * https://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +25,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef MYSQLX_COMMON_OP_IMPL_H
@@ -141,11 +141,15 @@ public:
 
   virtual ~Op_base() override
   {
-    // Let's aquire lock so that any remaingin replies are consumed on ~Reply
-    // and this way avoid race conditions with Client::close()
-    auto lock = m_sess->lock();
-    release_stmt_id();
-    m_reply.reset();
+    try {
+      // Let's aquire lock so that any remaingin replies are consumed on ~Reply
+      // and this way avoid race conditions with Client::close()
+      auto lock = m_sess->lock();
+      release_stmt_id();
+      m_reply.reset();
+    }
+    catch (...)
+    {}
   }
 
   cdk::Session& get_cdk_session()
@@ -160,8 +164,7 @@ public:
   uint32_t create_stmt_id()
   {
     assert(m_sess);
-    if (!m_stmt_id.unique())
-    {
+    if (m_stmt_id.use_count() != 1) {
       uint32_t id = m_sess->create_stmt_id();
       if(id != 0)
         m_stmt_id.reset(new uint32_t(id));
@@ -173,7 +176,7 @@ public:
 
   void release_stmt_id()
   {
-    if (m_stmt_id.unique())
+    if (m_stmt_id.use_count() == 1)
       m_sess->release_stmt_id(*m_stmt_id);
     m_stmt_id.reset();
   }
@@ -186,7 +189,7 @@ public:
 
   void reset_state()
   {
-    if (m_stmt_id.unique())
+    if (m_stmt_id.use_count()==1)
       get_session()->error_stmt_id(*m_stmt_id);
     m_stmt_id.reset();
     m_prepare_state = PS_EXECUTE;
