@@ -1,22 +1,22 @@
 /*
- * Copyright (c) 2017, 2020, Oracle and/or its affiliates.
+ * Copyright (c) 2017, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
  * published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
- * documentation.  The authors of MySQL hereby grant you an
- * additional permission to link the program and your derivative works
- * with the separately licensed software that they have included with
- * MySQL.
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation. The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * Without limiting anything contained in the foregoing, this file,
- * which is part of MySQL Connector/C++, is also subject to the
+ * which is part of Connector/C++, is also subject to the
  * Universal FOSS Exception, version 1.0, a copy of which can be found at
- * http://oss.oracle.com/licenses/universal-foss-exception.
+ * https://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +25,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "test.h"
@@ -66,7 +66,7 @@ TEST_F(Bugs,bug30989042_cdk_reply_error)
   // res.count() will consume the first row set, but another
   // one is pending.
 
-  EXPECT_EQ(1,res.count());
+  EXPECT_EQ(1U,res.count());
 
   // As we execute new statement, all remaining rows from the previous
   // reply should be cached. It was not the case before bug was fixed,
@@ -79,14 +79,14 @@ TEST_F(Bugs,bug30989042_cdk_reply_error)
   // and still available.
 
   EXPECT_TRUE(res.nextResult());
-  EXPECT_EQ(1,res.count());
+  EXPECT_EQ(1U,res.count());
 
   // Check that in case of error, it is reported only when moving to
   // the next result in the reply (should not be reported while accessing
   // the first result).
 
   res = sql("CALL test.p(1)");
-  EXPECT_EQ(1, res.count());
+  EXPECT_EQ(1U, res.count());
   EXPECT_THROW(res.nextResult(), Error);
 }
 
@@ -212,7 +212,7 @@ TEST_F(Bugs, bug_26962725_double_bind)
         .add(R"({"name":"johannes", "age":28})")
         .execute();
 
-  EXPECT_EQ(2, myColl.find().execute().count());
+  EXPECT_EQ(2U, myColl.find().execute().count());
 
   // Create Collection.remove() operation, but do not run it yet
   auto myRemove = myColl.remove("name = :param1 AND age = :param2");
@@ -221,7 +221,7 @@ TEST_F(Bugs, bug_26962725_double_bind)
   myRemove.bind("param1", "mike").bind("param2", 39).execute();
   myRemove.bind("param1", "johannes").bind("param2", 28).execute();
 
-  EXPECT_EQ(0, myColl.find().execute().count());
+  EXPECT_EQ(0U, myColl.find().execute().count());
 }
 
 TEST_F(Bugs, bug_27727505_multiple_results)
@@ -361,16 +361,39 @@ TEST_F(Bugs, bug_27727505_multiple_results)
     SqlResult res2 = sess.sql("call test").execute();
 
     //All resultsets are now cached
-    EXPECT_EQ(100, res.count());
+    EXPECT_EQ(100U, res.count());
     EXPECT_EQ(string("f0"), res.getColumn(0).getColumnName());
     EXPECT_TRUE(res.nextResult());
     EXPECT_EQ(static_cast<unsigned long>(0), res.count());
     EXPECT_EQ(string("f1"), res.getColumn(0).getColumnName());
     EXPECT_TRUE(res.nextResult());
-    EXPECT_EQ(11, res.count());
+    EXPECT_EQ(11U, res.count());
     EXPECT_EQ(string("new_f0"), res.getColumn(0).getColumnLabel());
     EXPECT_FALSE(res.nextResult());
   }
+
+
+  //This test uses undefined table, so an error should occur when getting to
+  //that result
+  sess.sql("drop procedure if exists test").execute();
+  sess.sql("CREATE PROCEDURE test() BEGIN select f0 from newtable; select f1 from newtable "
+           "where f0 > 100; select f0 as new_f0  from unknowntable where f0 <= 10;"
+           " END").execute();
+
+  {
+    SqlResult res = sess.sql("call test").execute();
+    EXPECT_EQ(100U, res.count());
+    EXPECT_EQ(string("f0"), res.getColumn(0).getColumnName());
+    std::vector<Row> rowAll = res.fetchAll();
+    EXPECT_EQ(100U, rowAll.size());
+    EXPECT_TRUE(res.nextResult());
+    EXPECT_EQ(0U, res.count());
+    EXPECT_EQ(string("f1"), res.getColumn(0).getColumnName());
+    rowAll = res.fetchAll();
+    EXPECT_EQ(0U, rowAll.size());
+    EXPECT_THROW(res.nextResult(), mysqlx::Error);
+    EXPECT_THROW(rowAll = res.fetchAll(), mysqlx::Error);
+  }
 }
 
 
@@ -443,7 +466,7 @@ TEST_F(Bugs, list_initializer)
                  ).execute();
 
   int count = 0;
-  for(const string& id : add_res.getGeneratedIds())
+  for(const std::string& id : add_res.getGeneratedIds())
   {
     std::cout << id << std::endl;
     ++count;
@@ -679,19 +702,19 @@ TEST_F(Bugs, not_accumulate)
   select.groupBy("doc->$.age");
 
   select.lockExclusive();
-  EXPECT_EQ(2, select.execute().count());
+  EXPECT_EQ(2U, select.execute().count());
 
   auto update = tbl.update();
   update.set("doc->$.age",1);
   update.where("doc->$.age > 7");
   update.orderBy("notfound ASC");
   update.orderBy("doc->$.age ASC");
-  EXPECT_EQ(1, update.execute().getAffectedItemsCount());
+  EXPECT_EQ(1U, update.execute().getAffectedItemsCount());
 
   auto tbl_remove = tbl.remove();
   tbl_remove.orderBy("notfound ASC");
   tbl_remove.orderBy("doc->$.age ASC");
-  EXPECT_EQ(2, tbl_remove.execute().getAffectedItemsCount());
+  EXPECT_EQ(2U, tbl_remove.execute().getAffectedItemsCount());
 }
 
 
@@ -721,7 +744,7 @@ TEST_F(Bugs, bug29525077)
   auto res = tab.select().execute();
   auto columns = &res.getColumns();
   col_count_t num_columns = res.getColumnCount();
-  EXPECT_EQ(10, num_columns);
+  EXPECT_EQ(10U, num_columns);
   while (Row r = res.fetchOne())
   {
     for (col_count_t i = 0; i < num_columns; ++i)
@@ -770,9 +793,9 @@ TEST_F(Bugs, is_false)
   // Had a segmentation fault issue
   EXPECT_THROW(coll.find("cast(val as boolean) is false").execute(),
                mysqlx::Error);
-  EXPECT_EQ(1, coll.find("val is false").execute().count());
+  EXPECT_EQ(1U, coll.find("val is false").execute().count());
   auto tbl = schema.getCollectionAsTable("is_false");
-  EXPECT_EQ(1, tbl.select().where("doc->$.val is false").execute().count());
+  EXPECT_EQ(1U, tbl.select().where("doc->$.val is false").execute().count());
 }
 
 TEST_F(Bugs, bug29394723)
@@ -853,10 +876,167 @@ TEST_F(Bugs, Bug31686958)
   auto find = coll.find("value = :value").bind("value", "1");
 
   //Succeeds, since its direct execute
-  EXPECT_EQ(1, find.execute().count());
+  EXPECT_EQ(1U, find.execute().count());
   //Should not fail do to being prepared+executed
-  EXPECT_EQ(1, find.execute().count());
-  EXPECT_EQ(1, find.execute().count());
+  EXPECT_EQ(1U, find.execute().count());
+  EXPECT_EQ(1U, find.execute().count());
 
 
 }
+
+TEST_F(Bugs, Bug29788255)
+{
+  SKIP_IF_NO_XPLUGIN;
+
+  SKIP_TEST("Untill Bug#29788255 is fixed");
+
+  auto &sess = get_sess();
+  auto sch = sess.createSchema("test", true);
+  auto coll = sch.createCollection("Bug29788255");
+
+  auto res = coll.find("1 overlaps (1,2)").execute();
+  EXPECT_EQ(1U, res.count());
+
+  res = coll.find("null overlaps [1]").execute();
+  EXPECT_EQ(0U, res.count());
+
+  res = coll.find("null overlaps [1, null]").execute();
+  EXPECT_EQ(1U, res.count());
+
+  res = coll.find("$.food overlaps [\"\"]").execute();
+  EXPECT_EQ(0U, res.count());
+
+  res = coll.find("$.food overlaps null").execute();
+  EXPECT_EQ(0U, res.count());
+
+  res = coll.find("$.food OVERLAPS \"@#$%^\"").execute();
+  EXPECT_EQ(0U, res.count());
+
+  res = coll.find("[\"@#$%^\"] OVERLAPS $.list").execute();
+  EXPECT_EQ(0U, res.count());
+
+  res = coll.find("(1+6) OVERLAPS [2,3,5]").execute();
+  EXPECT_EQ(0U, res.count());
+
+  res = coll.find("cast((1+6) AS JSON) OVERLAPS [2,3,7]").execute();
+  EXPECT_EQ(0U, res.count());
+
+  res = coll.find("[(1+6)] OVERLAPS [2,3,7]").execute();
+  EXPECT_EQ(1U, res.count());
+
+  res = coll.find("(1+6) in [2,3,7]").execute();
+  EXPECT_EQ(1U, res.count());
+}
+
+TEST_F(Bugs, Bug33352469)
+{
+  SKIP_IF_NO_XPLUGIN;
+
+  auto &sess = get_sess();
+  auto sch = sess.createSchema("test", true);
+  auto coll = sch.createCollection("Bug#33352469");
+
+  coll.remove("true").execute();
+  auto res = coll.add(R"({"_id":"1", "name": "user1"})")
+                 .add(R"({"_id":"2", "name": "user2"})")
+                 .execute();
+  EXPECT_EQ(2U, coll.find().execute().count());
+
+  auto remove = coll.remove(R"(_id=:id)");
+  remove.bind("id", "1").execute();
+  EXPECT_EQ(1U, coll.find().execute().count());
+  remove.bind("id", "2").execute();
+  EXPECT_EQ(0U, coll.find().execute().count());
+}
+
+TEST_F(Bugs, Bug35000027)
+{
+  auto coll = get_sess().getSchema("test").createCollection("coll", true);
+  coll.remove("true").execute();
+  size_t nr_docs = 10000;
+  DbDoc doc(R"({"name":"foo", "bar": "baz" })");
+  auto add_task = coll.add(doc);
+  auto add_tast_2 = add_task.add(doc);
+  for (size_t i = 2; i < nr_docs; ++i) {
+    add_task = add_task.add(doc);
+    add_tast_2 = add_tast_2.add(doc);
+  }
+  EXPECT_EQ(nr_docs, add_task.execute().getAffectedItemsCount());
+  EXPECT_EQ(nr_docs, add_tast_2.execute().getAffectedItemsCount());
+}
+
+TEST_F(Bugs, Bug35046616) {
+  Schema sch = get_sess().createSchema("test", true);
+  Collection coll = sch.createCollection("c1", true);
+
+  cout << "Inserting documents..." << endl;
+
+  coll.remove("true").execute();
+
+  auto ids = {"1", "MYID1", "MYID2"};
+
+  for (auto _id : ids) {
+    coll.add(std::string(R"({ "_id": ")") + _id +
+             R"(", "name": "foo", "age": 1 })")
+        .execute();
+  }
+
+  auto find_id = coll.find("_id = :id");
+  for(auto _id : ids)
+  {
+    EXPECT_EQ(std::string("foo"), find_id.bind("id", _id)
+                                      .execute()
+                                      .fetchOne()["name"]
+                                      .get<std::string>());
+  }
+
+  cout << "Updating document..." << endl;
+
+  auto modify = coll.modify("_id = :id")
+                    .set("$",expr( R"({ "_id": "DISCARDED", "name": "bar"})"));
+  for (auto _id : ids) {
+    modify.bind("id", _id).execute();
+
+    EXPECT_EQ(std::string("bar"), find_id.bind("id", _id)
+                                      .execute()
+                                      .fetchOne()["name"]
+                                      .get<std::string>());
+  }
+
+  // Changing all documents of the collection setting name to baz
+  coll.modify("true")
+      .set("$", DbDoc(R"({ "_id": "DISCARDED", "name": "baz"})"))
+      .execute();
+
+  for (auto _id : ids) {
+    EXPECT_EQ(std::string("baz"), find_id.bind("id", _id)
+                                      .execute()
+                                      .fetchOne()["name"]
+                                      .get<std::string>());
+  }
+
+  // Changing all documents of the collection setting name to a document
+  // {first: last:}
+  coll.modify("true")
+      .set("$.name", DbDoc(R"({"first": "foo", "last": "bar"})"))
+      .execute();
+
+  for (auto _id : ids) {
+    auto doc = find_id.bind("id", _id).execute().fetchOne();
+    EXPECT_EQ(Value::DOCUMENT, doc["name"].getType());
+    EXPECT_EQ(std::string("foo"), doc["name"]["first"].get<std::string>());
+  }
+
+  // Changing all documents of the collection setting name to a string with a
+  // JSON. JSON should not be processed.
+  coll.modify("true")
+      .set("$.name", R"({"first": "foo", "last": "bar"})")
+      .execute();
+
+  for (auto _id : ids) {
+    auto doc = find_id.bind("id", _id).execute().fetchOne();
+    EXPECT_EQ(Value::STRING, doc["name"].getType());
+    EXPECT_EQ(std::string(R"({"first": "foo", "last": "bar"})"),
+              doc["name"].get<std::string>());
+  }
+}
\ No newline at end of file
