@@ -1,22 +1,22 @@
 /*
- * Copyright (c) 2015, 2020, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
  * published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
- * documentation.  The authors of MySQL hereby grant you an
- * additional permission to link the program and your derivative works
- * with the separately licensed software that they have included with
- * MySQL.
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation. The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * Without limiting anything contained in the foregoing, this file,
- * which is part of MySQL Connector/C++, is also subject to the
+ * which is part of Connector/C++, is also subject to the
  * Universal FOSS Exception, version 1.0, a copy of which can be found at
- * http://oss.oracle.com/licenses/universal-foss-exception.
+ * https://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +25,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <iostream>
@@ -55,23 +55,23 @@ TEST_F(Types, numeric)
     Value val = 7U;
     EXPECT_EQ(Value::UINT64, val.getType());
 
-    int v0;
+    int v0 = 0;
     EXPECT_NO_THROW(v0 = val);
     EXPECT_EQ(7, v0);
 
-    unsigned v1;
+    unsigned v1 = 0U;
     EXPECT_NO_THROW(v1 = val);
-    EXPECT_EQ(7, v1);
+    EXPECT_EQ(7U, v1);
 
-    float v2;
+    float v2 = 0.0;
     EXPECT_NO_THROW(v2 = val);
     EXPECT_EQ(7, v2);
 
-    double v3;
+    double v3 = 0.0;
     EXPECT_NO_THROW(v3 = val);
     EXPECT_EQ(7, v3);
 
-    bool v4;
+    bool v4 = false;
     EXPECT_NO_THROW(v4 = (bool)val);
     EXPECT_TRUE(v4);
   }
@@ -80,22 +80,21 @@ TEST_F(Types, numeric)
     Value val = -7;
     EXPECT_EQ(Value::INT64, val.getType());
 
-    int v0;
+    int v0=0;
     EXPECT_NO_THROW(v0 = val);
     EXPECT_EQ(-7, v0);
 
-    unsigned v1;
-    EXPECT_THROW(v1 = val, Error);
+    EXPECT_THROW((unsigned)val, Error);
 
-    float v2;
+    float v2=0.0;
     EXPECT_NO_THROW(v2 = val);
     EXPECT_EQ(-7, v2);
 
-    double v3;
+    double v3=0.0;
     EXPECT_NO_THROW(v3 = val);
     EXPECT_EQ(-7, v3);
 
-    bool v4;
+    bool v4=false;
     EXPECT_NO_THROW(v4 = (bool)val);
     EXPECT_TRUE(v4);
   }
@@ -105,23 +104,22 @@ TEST_F(Types, numeric)
     Value val = max_uint;
     EXPECT_EQ(Value::UINT64, val.getType());
 
-    int v0;
-    EXPECT_THROW(v0 = val, Error);
+    EXPECT_THROW((int)val, Error);
 
-    unsigned v1;
+    unsigned v1=0;
     EXPECT_NO_THROW(v1 = val);
     EXPECT_EQ(max_uint, v1);
 
-    float v2;
+    float v2=0.0;
     EXPECT_NO_THROW(v2 = val);
     // Note: allow small rounding errors
     EXPECT_LE(fabs(v2/max_uint - 1), 1e-7);
 
-    double v3;
+    double v3=0.0;
     EXPECT_NO_THROW(v3 = val);
     EXPECT_EQ(max_uint, v3);
 
-    bool v4;
+    bool v4=false;
     EXPECT_NO_THROW(v4 = (bool)val);
     EXPECT_TRUE(v4);
   }
@@ -130,11 +128,9 @@ TEST_F(Types, numeric)
     Value val = 7.0F;
     EXPECT_EQ(Value::FLOAT, val.getType());
 
-    int v0;
-    EXPECT_THROW(v0 = val, Error);
+    EXPECT_THROW(static_cast<int>(val), Error);
 
-    unsigned v1;
-    EXPECT_THROW(v1 = val, Error);
+    EXPECT_THROW(static_cast<unsigned>(val), Error);
 
     float v2;
     EXPECT_NO_THROW(v2 = val);
@@ -144,29 +140,24 @@ TEST_F(Types, numeric)
     EXPECT_NO_THROW(v3 = val);
     EXPECT_EQ(7.0, v3);
 
-    bool v4;
-    EXPECT_THROW(v4 = (bool)val, Error);
+    EXPECT_THROW(static_cast<bool>(val), Error);
   }
 
   {
     Value val = 7.0;
     EXPECT_EQ(Value::DOUBLE, val.getType());
 
-    int v0;
-    EXPECT_THROW(v0 = val, Error);
+    EXPECT_THROW(static_cast<int>(val), Error);
 
-    unsigned v1;
-    EXPECT_THROW(v1 = val, Error);
+    EXPECT_THROW(static_cast<unsigned>(val), Error);
 
-    float v2;
-    EXPECT_THROW(v2 = val, Error);
+    EXPECT_THROW(static_cast<float>(val), Error);
 
     double v3;
     EXPECT_NO_THROW(v3 = val);
     EXPECT_EQ(7.0, v3);
 
-    bool v4;
-    EXPECT_THROW(v4 = (bool)val, Error);
+    EXPECT_THROW(static_cast<bool>(val), Error);
   }
 
   {
@@ -177,17 +168,15 @@ TEST_F(Types, numeric)
     EXPECT_NO_THROW(v0 = val);
     EXPECT_EQ(1, v0);
 
-    unsigned v1;
+    unsigned v1=0;
     EXPECT_NO_THROW(v1 = val);
-    EXPECT_EQ(1, v1);
+    EXPECT_EQ(1U, v1);
 
-    float v2;
-    EXPECT_THROW(v2 = val, Error);
+    EXPECT_THROW(static_cast<float>(val), Error);
 
-    double v3;
-    EXPECT_THROW(v3 = val, Error);
+    EXPECT_THROW(static_cast<double>(val), Error);
 
-    bool v4;
+    bool v4=false;
     EXPECT_NO_THROW(v4 = (bool)val);
     EXPECT_TRUE(v4);
   }
@@ -286,7 +275,7 @@ TEST_F(Types, basic)
   EXPECT_EQ(Type::BIT, c5.getType());
   cout << "column " << res.getColumn(5) << " length: "
     << c5.getLength();
-  EXPECT_EQ(64, c5.getLength());
+  EXPECT_EQ(64U, c5.getLength());
 
   for (unsigned i = 0; (row = res.fetchOne()); ++i)
   {
@@ -312,7 +301,7 @@ TEST_F(Types, basic)
     EXPECT_EQ(data_string[i], (string)row[4]);
     EXPECT_EQ(data_bit[i], (uint64_t)row[5]);
 
-    EXPECT_GT(row[1].getRawBytes().size(), 1);
+    EXPECT_GT(row[1].getRawBytes().size(), 1U);
     EXPECT_EQ(data_string[i].length(), string(row[4]).length());
 
   }
@@ -389,15 +378,15 @@ TEST_F(Types, integer)
   {
     Value v1(7U);
     EXPECT_EQ(Value::UINT64, v1.getType());
-    EXPECT_EQ(7, (uint64_t)v1);
+    EXPECT_EQ(7U, (uint64_t)v1);
 
     Value v2(7UL);
     EXPECT_EQ(Value::UINT64, v1.getType());
-    EXPECT_EQ(7, (uint64_t)v1);
+    EXPECT_EQ(7U, (uint64_t)v1);
 
     Value v3(7ULL);
     EXPECT_EQ(Value::UINT64, v1.getType());
-    EXPECT_EQ(7, (uint64_t)v1);
+    EXPECT_EQ(7U, (uint64_t)v1);
   }
 
   SKIP_IF_NO_XPLUGIN;
@@ -471,7 +460,7 @@ TEST_F(Types, string)
   cout << "column #0 charset: " << c0.getCharacterSetName() << endl;
   cout << "column #0 collation: " << c0.getCollationName() << endl;
 
-  EXPECT_EQ(10, c0.getLength());
+  EXPECT_EQ(10U, c0.getLength());
   //EXPECT_EQ(CharacterSet::latin2, c0.getCharacterSet());
   //EXPECT_EQ(Collation<CharacterSet::latin2>::general_ci, c0.getCollation());
 
@@ -551,6 +540,8 @@ TEST_F(Types, collations)
 {
   SKIP_IF_NO_XPLUGIN;
 
+  SKIP_IF_SERVER_VERSION_LESS(8, 0, 30);
+
   using col_data = std::pair<unsigned, string>;
   std::vector<col_data> unknown;
 
@@ -760,7 +751,7 @@ TEST_F(Types, json)
   EXPECT_EQ(Type::JSON, c0.getType());
 
   Row row;
-  for (unsigned i = 0; (row = res.fetchOne()); ++i)
+  while((row = res.fetchOne()))
   {
     EXPECT_EQ(Value::DOCUMENT, row[0].getType());
 
@@ -776,7 +767,7 @@ TEST_F(Types, json)
     EXPECT_EQ(Value::DOCUMENT, doc["sub"].getType());
 
     EXPECT_EQ(7, (int)doc["foo"]);
-    EXPECT_EQ(3, doc["arr"].elementCount());
+    EXPECT_EQ(3U, doc["arr"].elementCount());
     EXPECT_TRUE(doc["sub"].hasField("day"));
     EXPECT_TRUE(doc["sub"].hasField("month"));
   }
@@ -826,8 +817,10 @@ TEST_F(Types, json)
       EXPECT_EQ(1, (int)row[0][0]);
       EXPECT_EQ(string("a"), (string)row[0][1]);
 
-      if (0 == i)
+      if (0U == i)
+      {
         EXPECT_EQ(Value::DOCUMENT, row[0][2].getType());
+      }
     }
   }
 
@@ -965,11 +958,11 @@ TEST_F(Types, datetime)
     {
     case Type::DATE:
     case Type::TIME:
-      EXPECT_EQ(4, row[j].getRawBytes().size());
+      EXPECT_EQ(4U, row[j].getRawBytes().size());
       break;
     case Type::DATETIME:
     case Type::TIMESTAMP:
-      EXPECT_EQ(6, row[j].getRawBytes().size());
+      EXPECT_EQ(6U, row[j].getRawBytes().size());
       break;
     default:
       FAIL() << "Unexpected type! Update UT";
@@ -1099,8 +1092,7 @@ TEST_F(Types, int64_conversion)
   EXPECT_EQ(std::numeric_limits<int64_t>::max(), int64_v);
 
   //should overflow
-  int int_v;
-  EXPECT_ANY_THROW(int_v = value);
+  EXPECT_ANY_THROW(static_cast<int>(value));
 
   //now with min value
   value = std::numeric_limits<int64_t>::min();
@@ -1109,7 +1101,7 @@ TEST_F(Types, int64_conversion)
 
   EXPECT_EQ(std::numeric_limits<int64_t>::min(), int64_v);
 
-  EXPECT_ANY_THROW(int_v = value);
+  EXPECT_ANY_THROW(static_cast<int>(value));
 
   //Now using the uint64_t max.
   value = std::numeric_limits<uint64_t>::max();
@@ -1120,6 +1112,5 @@ TEST_F(Types, int64_conversion)
 
   EXPECT_ANY_THROW(int64_v = value);
 
-  EXPECT_ANY_THROW(int_v = value);
-
+  EXPECT_ANY_THROW(static_cast<int>(value));
 }
