@@ -1,22 +1,22 @@
 /*
- * Copyright (c) 2008, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
  * published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
- * documentation.  The authors of MySQL hereby grant you an
- * additional permission to link the program and your derivative works
- * with the separately licensed software that they have included with
- * MySQL.
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation. The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * Without limiting anything contained in the foregoing, this file,
- * which is part of MySQL Connector/C++, is also subject to the
+ * which is part of Connector/C++, is also subject to the
  * Universal FOSS Exception, version 1.0, a copy of which can be found at
- * http://oss.oracle.com/licenses/universal-foss-exception.
+ * https://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +25,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
@@ -202,11 +202,18 @@ MyVal::getBool()
 
 
 /* {{{ MySQL_ArtResultSet::MySQL_ArtResultSet() -I- */
-MySQL_ArtResultSet::MySQL_ArtResultSet(const StringList& fn, boost::shared_ptr< rset_t > &rs, boost::shared_ptr< MySQL_DebugLogger > & l)
-  : num_fields(static_cast<int>(fn.size())), rset(rs), current_record(rset->begin()),
-    started(false), field_index_to_name_map(new sql::SQLString[num_fields]),
-  num_rows(rset->size()), row_position(0), is_closed(false), logger(l)
-{
+MySQL_ArtResultSet::MySQL_ArtResultSet(const StringList &fn,
+                                       std::shared_ptr<rset_t> &rs,
+                                       std::shared_ptr<MySQL_DebugLogger> &l)
+    : num_fields(static_cast<int>(fn.size())),
+      rset(rs),
+      current_record(rset->begin()),
+      started(false),
+      field_index_to_name_map(new sql::SQLString[num_fields]),
+      num_rows(rset->size()),
+      row_position(0),
+      is_closed(false),
+      logger(l) {
   CPP_ENTER("MySQL_ArtResultSet::MySQL_ArtResultSet");
   CPP_INFO_FMT("metadata.size=%d resultset.size=%d", fn.size(), rset->size());
 
@@ -257,15 +264,20 @@ MySQL_ArtResultSet::isBeforeFirstOrAfterLast() const
 }
 /* }}} */
 
+/* {{{ MySQL_ArtResultSet::absolute() -I-
+  _new_pos: Seek the passed position on the ResultSet. If _new_pos is negative,
+            it will count from last to beginning.
 
-/* {{{ MySQL_ArtResultSet::absolute() -I- */
+  return: true if position is a valid row, false if it pasted last or beginnin
+*/
 bool
-MySQL_ArtResultSet::absolute(const int row)
+MySQL_ArtResultSet::absolute(const int _row)
 {
   CPP_ENTER("MySQL_ArtResultSet::absolute");
   checkValid();
+  const int64_t row = _row;
   if (row > 0) {
-    if (row > (int) num_rows) {
+    if (static_cast<uint64_t>(row) > num_rows) {
       afterLast();
     } else {
       row_position = row;
@@ -273,10 +285,10 @@ MySQL_ArtResultSet::absolute(const int row)
       return true;
     }
   } else if (row < 0) {
-    if ((-row) > (int) num_rows) {
+    if (static_cast<uint64_t>(-row) > num_rows) {
       beforeFirst();
     } else {
-      row_position = num_rows - (-row)  + 1;
+      row_position = num_rows + row + 1;
       seek();
       return true;
     }
@@ -722,6 +734,28 @@ MySQL_ArtResultSet::getString(const sql::SQLString& columnLabel) const
 /* }}} */
 
 
+/* {{{ MySQL_ArtResultSet::getVector() -U- */
+std::vector<float>
+MySQL_ArtResultSet::getVector(uint32_t columnIndex) const
+{
+  CPP_ENTER("MySQL_ArtResultSet::getVector");
+  checkValid();
+  throw sql::MethodNotImplementedException("MySQL_ArtResultSet::getVector()");
+}
+/* }}} */
+
+
+/* {{{ MySQL_ArtResultSet::getVector() -U- */
+std::vector<float>
+MySQL_ArtResultSet::getVector(const sql::SQLString& columnLabel) const
+{
+  CPP_ENTER("MySQL_ArtResultSet::getVector");
+  checkValid();
+  throw sql::MethodNotImplementedException("MySQL_ArtResultSet::getVector()");
+}
+/* }}} */
+
+
 /* {{{ MySQL_ArtResultSet::getType() -I- */
 sql::ResultSet::enum_type
 MySQL_ArtResultSet::getType() const
