@@ -1,22 +1,22 @@
 /*
- * Copyright (c) 2008, 2022, Oracle and/or its affiliates.
+ * Copyright (c) 2008, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
  * published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
- * documentation.  The authors of MySQL hereby grant you an
- * additional permission to link the program and your derivative works
- * with the separately licensed software that they have included with
- * MySQL.
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation. The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * Without limiting anything contained in the foregoing, this file,
- * which is part of MySQL Connector/C++, is also subject to the
+ * which is part of Connector/C++, is also subject to the
  * Universal FOSS Exception, version 1.0, a copy of which can be found at
- * http://oss.oracle.com/licenses/universal-foss-exception.
+ * https://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +25,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
@@ -34,6 +34,7 @@
 #include <algorithm>
 #include <sstream>
 #include <cstring>
+#include <cassert>
 
 
 /*
@@ -46,6 +47,7 @@
 #include <cppconn/resultset.h>
 #include <cppconn/warning.h>
 #include "mysql_connection.h"
+#include "mysql_connection_data.h"
 #include "mysql_statement.h"
 #include "mysql_resultset.h"
 #include "mysql_warning.h"
@@ -104,22 +106,68 @@ MySQL_Statement::do_query(const ::sql::SQLString &q)
     throw sql::InvalidInstanceException("Connection has been closed");
   }
 
-  if(attrbind.nrAttr() != 0)
-  {
-    proxy_p->query_attr(attrbind.nrAttr(), attrbind.getNames(), attrbind.getBinds());
-  }
+  telemetry.span_start(this);
 
-  if (proxy_p->query(q) && proxy_p->errNo()) {
-    CPP_ERR_FMT("Error during proxy->query : %d:(%s) %s", proxy_p->errNo(), proxy_p->sqlstate().c_str(), proxy_p->error().c_str());
-    sql::mysql::util::throwSQLException(*proxy_p.get());
+  try
+  {
+    if(attrbind.size() != 0)
+    {
+      proxy_p->query_attr(attrbind.size(), attrbind.getNames(), attrbind.getBinds());
+    }
+
+    if (proxy_p->query(q) && proxy_p->errNo()) {
+      CPP_ERR_FMT("Error during proxy->query : %d:(%s) %s", proxy_p->errNo(), proxy_p->sqlstate().c_str(), proxy_p->error().c_str());
+
+      sql::mysql::util::throwSQLException(*proxy_p.get());
+    }
+
+    // Note: If statement has no results then we close the span here, otherwise
+    // it will be closed after reading all result sets.
+
+    if ((0 == proxy_p->field_count()) && !proxy_p->more_results())
+      telemetry.span_end(this);
+  }
+  catch(sql::SQLException &e)
+  {
+    telemetry.set_error(this, e.what());
+    throw;
   }
 
   warningsCount= proxy_p->warning_count();
-
   warningsHaveBeenLoaded= false;
 }
 /* }}} */
 
+telemetry::Telemetry<MySQL_Connection>&
+MySQL_Statement::conn_telemetry()
+{
+  assert(connection);
+
+  /*
+    Note: It can happend that connection was closed when this method is called.
+    In that case connection->intern is empty and we have nothing to return
+    here. Then we use a dummy telemetry object which is always disabled.
+  */
+
+  static telemetry::Telemetry<MySQL_Connection> dummy{OTEL_DISABLED};
+
+  /*
+    Note: `proxy` is a weak pointer to the proxy object owned by
+    the connection. If connection has been deleted then `proxy` should
+    be expired and we use this fact to detect whether the connection
+    is still good.
+  */
+
+  if (proxy.expired())
+    return dummy;
+
+  // Note: We assume that if the proxy is still around then so is
+  // the connection and it has non-empty intern pointer.
+
+  assert(connection->intern);
+  return connection->intern->telemetry;
+}
+
 
 /* {{{ MySQL_Statement::get_resultset() -I- */
 std::shared_ptr<NativeAPI::NativeResultsetWrapper>
@@ -171,6 +219,7 @@ MySQL_Statement::cancel()
 /* {{{ MySQL_Statement::execute() -I- */
 bool
 MySQL_Statement::execute(const sql::SQLString& sql)
+try
 {
   CPP_ENTER("MySQL_Statement::execute");
   CPP_INFO_FMT("this=%p", this);
@@ -185,12 +234,23 @@ MySQL_Statement::execute(const sql::SQLString& sql)
   last_update_count = ret? UL64(~0):proxy_p->affected_rows();
   return ret;
 }
+catch (sql::SQLException &e)
+{
+  telemetry.set_error(this, e.what());
+  throw e;
+}
+catch (...)
+{
+  telemetry.set_error(this, "Unknown error in MySQL_Statement::execute");
+  throw;
+}
 /* }}} */
 
 
 /* {{{ MySQL_Statement::executeQuery() -I- */
 sql::ResultSet *
 MySQL_Statement::executeQuery(const sql::SQLString& sql)
+try
 {
   CPP_ENTER("MySQL_Statement::executeQuery");
   CPP_INFO_FMT("this=%p", this);
@@ -209,6 +269,16 @@ MySQL_Statement::executeQuery(const sql::SQLString& sql)
   CPP_INFO_FMT("rset=%p", tmp);
   return tmp;
 }
+catch (sql::SQLException &e)
+{
+  telemetry.set_error(this, e.what());
+  throw e;
+}
+catch (...)
+{
+  telemetry.set_error(this, "Unknown error in MySQL_Statement::executeQuery");
+  throw;
+}
 /* }}} */
 
 
@@ -224,6 +294,7 @@ dirty_drop_rs(std::shared_ptr< NativeAPI::NativeConnectionWrapper > proxy)
 /* {{{ MySQL_Statement::executeUpdate() -I- */
 int
 MySQL_Statement::executeUpdate(const sql::SQLString& sql)
+try
 {
   CPP_ENTER("MySQL_Statement::executeUpdate");
   CPP_INFO_FMT("this=%p", this);
@@ -253,6 +324,7 @@ MySQL_Statement::executeUpdate(const sql::SQLString& sql)
       if (got_rs){
         throw sql::InvalidArgumentException("Statement returning result set");
       } else {
+        telemetry.span_end(this);
         return static_cast<int>(last_update_count);
       }
     }
@@ -270,8 +342,20 @@ MySQL_Statement::executeUpdate(const sql::SQLString& sql)
   } while (1);
 
   /* Should not actually get here*/
+  telemetry.span_end(this);
   return 0;
 }
+catch (sql::SQLException &e)
+{
+  telemetry.set_error(this, e.what());
+  throw e;
+}
+catch (...)
+{
+  telemetry.set_error(this, "Unknown error in MySQL_Statement::executeUpdate");
+  throw;
+}
+
 /* }}} */
 
 
@@ -303,6 +387,7 @@ MySQL_Statement::getFetchSize()
 /* {{{ MySQL_Statement::getResultSet() -I- */
 sql::ResultSet *
 MySQL_Statement::getResultSet()
+try
 {
   CPP_ENTER("MySQL_Statement::getResultSet");
   CPP_INFO_FMT("this=%p", this);
@@ -346,12 +431,15 @@ MySQL_Statement::getResultSet()
     }
     else
     {
+      // End span for NULL resultset
+      telemetry.span_end(this);
       return NULL;
     }
   }
 
   if (!result) {
     /* if there was an update then this method should return NULL and not throw */
+    telemetry.span_end(this);
     return NULL;
   }
 
@@ -360,6 +448,17 @@ MySQL_Statement::getResultSet()
   CPP_INFO_FMT("res=%p", ret);
   return ret;
 }
+catch (sql::SQLException &e)
+{
+  telemetry.set_error(this, e.what());
+  throw e;
+}
+catch (...)
+{
+  telemetry.set_error(this, "Unknown error in MySQL_Statement::getResultSet");
+  throw;
+}
+
 /* }}} */
 
 
@@ -441,6 +540,7 @@ MySQL_Statement::getMaxRows()
 /* {{{ MySQL_Statement::getMoreResults() -I- */
 bool
 MySQL_Statement::getMoreResults()
+try
 {
   CPP_ENTER("MySQL_Statement::getMoreResults");
   CPP_INFO_FMT("this=%p", this);
@@ -467,8 +567,21 @@ MySQL_Statement::getMoreResults()
       throw sql::SQLException("Impossible! more_results() said true, next_result says no more results");
     }
   }
+  // If no more results close the statement span
+  telemetry.span_end(this);
   return false;
 }
+catch (sql::SQLException &e)
+{
+  telemetry.set_error(this, e.what());
+  throw e;
+}
+catch (...)
+{
+  telemetry.set_error(this, "Unknown error in MySQL_Statement::getMoreResults");
+  throw;
+}
+
 /* }}} */
 
 
@@ -802,6 +915,14 @@ MySQL_Statement::setQueryAttrString(const sql::SQLString &name, const sql::SQLSt
 /* }}} */
 
 
+int setStmtAttrString(MySQL_Statement &stmt,
+  const sql::SQLString &name, const sql::SQLString &value,
+  bool is_external)
+{
+  return stmt.attrbind.setQueryAttrString(name, value, is_external);
+}
+
+
 /* {{{ MySQL_Statement::clearAttributes() -U- */
 void
 MySQL_Statement::clearAttributes()
