@@ -34,6 +34,7 @@
 #include <cppconn/exception.h>
 
 #include <memory>
+#include <map>
 
 #include "../mysql_util.h"
 #include "../mysql_connection_options.h"
@@ -52,6 +53,9 @@ namespace mysql
 namespace NativeAPI
 {
 
+std::shared_mutex plugins_cache_mutex;
+std::map <::sql::SQLString, st_mysql_client_plugin*> plugins_cache;
+
 /*
   Function to convert sql::mysql::MySQL_Connection_Options to
   libmysqlclient mysql_option
@@ -76,7 +80,16 @@ get_mysql_option(sql::mysql::MySQL_Connection_Options opt)
   case sql::mysql::MYSQL_OPT_WRITE_TIMEOUT: return ::MYSQL_OPT_WRITE_TIMEOUT;
   case sql::mysql::MYSQL_OPT_USE_RESULT: return ::MYSQL_OPT_USE_RESULT;
   case sql::mysql::MYSQL_REPORT_DATA_TRUNCATION: return ::MYSQL_REPORT_DATA_TRUNCATION;
-  case sql::mysql::MYSQL_OPT_RECONNECT: return ::MYSQL_OPT_RECONNECT;
+  case sql::mysql::MYSQL_OPT_RECONNECT:
+#if MYCPPCONN_STATIC_MYSQL_VERSION_ID < 80400
+    return ::MYSQL_OPT_RECONNECT;
+#else
+  {
+    std::string errorOption("MYSQL_OPT_RECONNECT");
+    throw sql::SQLUnsupportedOptionException("Option is not supported",
+      errorOption);
+  }
+#endif
   case sql::mysql::MYSQL_PLUGIN_DIR: return ::MYSQL_PLUGIN_DIR;
   case sql::mysql::MYSQL_DEFAULT_AUTH: return ::MYSQL_DEFAULT_AUTH;
   case sql::mysql::MYSQL_OPT_BIND: return ::MYSQL_OPT_BIND;
@@ -177,6 +190,15 @@ MySQL_NativeConnectionWrapper::connect(const ::sql::SQLString & host,
                   const ::sql::SQLString & socket_or_pipe,
                   unsigned long			client_flag)
 {
+  m_host = host;
+  m_user = user;
+  m_passwd = passwd;
+  m_db = db;
+  m_port = port;
+  m_socket_or_pipe = socket_or_pipe;
+  m_client_flag = client_flag;
+  m_dns_srv = false;
+
   return (NULL != api->real_connect(mysql, nullIfEmpty(host), user.c_str(),
                   nullIfEmpty(passwd),
                   nullIfEmpty(db), port,
@@ -193,6 +215,13 @@ MySQL_NativeConnectionWrapper::connect_dns_srv(const ::sql::SQLString & host,
                   const ::sql::SQLString & db,
                   unsigned long			client_flag)
 {
+  m_host = host;
+  m_user = user;
+  m_passwd = passwd;
+  m_db = db;
+  m_client_flag = client_flag;
+  m_dns_srv = true;
+
   return (NULL != api->real_connect_dns_srv(mysql, nullIfEmpty(host), user.c_str(),
                   nullIfEmpty(passwd),
                   nullIfEmpty(db), client_flag));
@@ -318,6 +347,13 @@ MySQL_NativeConnectionWrapper::next_result()
 int
 MySQL_NativeConnectionWrapper::options(::sql::mysql::MySQL_Connection_Options option, const void * value)
 {
+#if MYCPPCONN_STATIC_MYSQL_VERSION_ID >= 80400
+  if (option == MYSQL_OPT_RECONNECT) {
+    reconnect = *(bool*)value;
+    // For reconnect option we don't pass the call to api.
+    return 0;
+  }
+#endif
   return api->options(mysql, get_mysql_option(option), value);
 }
 /* }}} */
@@ -378,6 +414,14 @@ MySQL_NativeConnectionWrapper::options(::sql::mysql::MySQL_Connection_Options op
 int
 MySQL_NativeConnectionWrapper::get_option(::sql::mysql::MySQL_Connection_Options option, const void * value)
 {
+#if MYCPPCONN_STATIC_MYSQL_VERSION_ID >= 80400
+  if (option == MYSQL_OPT_RECONNECT) {
+    *(bool*)value = reconnect;
+    // For reconnect option we don't pass the call to api.
+    return 0;
+  }
+#endif
+
   return api->get_option(mysql, get_mysql_option(option), value);
 }
 /* }}} */
@@ -412,21 +456,143 @@ MySQL_NativeConnectionWrapper::get_option(::sql::mysql::MySQL_Connection_Options
 }
 /* }}} */
 
+
+/*
+  Accessing plugin options.
+*/
+
+void MySQL_NativeConnectionWrapper::lock_plugin_impl(lock_type type)
+{
+  switch (type)
+  {
+    case lock_type::GUARD:
+    case lock_type::SHARED:
+      // Note: We don't take shared lock if exclusive lock is in place
+      if (!plugins_ex_lock.owns_lock() && !plugins_sh_lock.owns_lock())
+        plugins_sh_lock.lock();
+
+      if (!plugin_guard)
+        plugin_guard = (lock_type::GUARD == type);
+    break;
+
+    case lock_type::EXCLUSIVE:
+      // Nothing to do if we already hold an exclusive lock
+      if (plugins_ex_lock.owns_lock())
+        break;
+      // Before taking exclusive lock we need to release the shared lock
+      if (plugins_sh_lock.owns_lock())
+        plugins_sh_lock.unlock();
+      // This will wait until no other connection holds a lock
+      plugins_ex_lock.lock();
+    break;
+
+    case lock_type::UNLOCK:
+      // UNLOCK has no effect if GUARD lock was taken
+      if (plugin_guard)
+        break;
+      // Otherwise UNLOCK and UNGUARD are the same
+      // FALLTHROUGH
+    case lock_type::UNGUARD:
+      if (plugins_sh_lock.owns_lock())
+        plugins_sh_lock.unlock();
+      if (plugins_ex_lock.owns_lock())
+        plugins_ex_lock.unlock();
+    break;
+  }
+}
+
+struct MySQL_NativeConnectionWrapper::PluginGuard
+{
+  using ConWrapper = MySQL_NativeConnectionWrapper;
+
+  ConWrapper *conn;
+
+  PluginGuard(ConWrapper *c)
+  : conn{c}
+  {
+    conn->lock_plugin_impl(lock_type::SHARED);
+  }
+
+  ~PluginGuard()
+  {
+    // Note: This will do nothing if external guard was created
+    conn->lock_plugin_impl(lock_type::UNLOCK);
+  }
+
+  // Upgrade initial shared lock to exclusive one
+
+  void lock()
+  {
+    conn->lock_plugin_impl(lock_type::EXCLUSIVE);
+  }
+
+  /*
+    Find a plugin in the plugin cache. If `load` is true then loads the plugin
+    into cache if not already there; otherwise null is returned if the plugin
+    was not found.
+
+    Throws error if loading of the plugin failed.
+  */
+
+  st_mysql_client_plugin*
+  get_plugin(int plugin_type, SQLString const &name, bool load)
+  {
+    /*
+      Note: Shared plugin lock is held when this method is called (taken in
+      the constructor).
+    */
+
+    if (plugins_cache.count(name) > 0)
+      return plugins_cache.at(name);
+    else if (load)
+    {
+      /* load client authentication plugin if required */
+      auto *plugin = conn->api->client_find_plugin(conn->mysql, name.c_str(), plugin_type);
+
+      lock();  // Upgrade to exclusive lock before writing to cache
+
+      plugins_cache.emplace(name, plugin);
+
+      return plugin;
+    }
+    else
+      return nullptr;
+  }
+};
+
+
 int
 MySQL_NativeConnectionWrapper::plugin_option(
-    int plugin_type,
-    const ::sql::SQLString & plugin_name,
-    const ::sql::SQLString & option,
-    const void * value)
-try{
+  int plugin_type,
+  const ::sql::SQLString & plugin_name,
+  const ::sql::SQLString & option,
+  const void * value,
+  bool default_value
+)
+try
+{
+  PluginGuard guard{this};
 
-  /* load client authentication plugin if required */
-  struct st_mysql_client_plugin *plugin =
-      api->client_find_plugin(mysql, plugin_name.c_str(), plugin_type);
+  /*
+    Note: Try to load plugin into cache only if the option is set to
+    a non-default value.
+  */
 
-  /* set option value in plugin */
+  struct st_mysql_client_plugin *plugin
+  = guard.get_plugin(plugin_type, plugin_name, !default_value);
+
+  /*
+    Note: `plugin` can be null here only if we are setting option to
+    the default value and the plugin was not found in the cache. Otherwise
+    an attempt to load the plugin will be made and error will be thrown if
+    it could not be done.
+  */
+
+  if (!plugin)
+    return 0;
+
+  guard.lock();  // Note: Only now an exclusive lock is required
   return api->plugin_options(plugin, option.c_str(), value);
-
 }
 catch(sql::InvalidArgumentException &e)
 {
@@ -435,41 +601,32 @@ catch(sql::InvalidArgumentException &e)
   throw sql::InvalidArgumentException(err);
 }
 
+
 int
 MySQL_NativeConnectionWrapper::plugin_option(
-    int plugin_type,
-    const ::sql::SQLString & plugin_name,
-    const ::sql::SQLString & option,
-    const ::sql::SQLString & value)
-try{
-
-  /* load client authentication plugin if required */
-  struct st_mysql_client_plugin *plugin =
-      api->client_find_plugin(mysql, plugin_name.c_str(), plugin_type);
-
-  /* set option value in plugin */
-  return api->plugin_options(plugin, option.c_str(), value.c_str());
-
-}
-catch(sql::InvalidArgumentException &e)
+  int plugin_type,
+  const ::sql::SQLString & plugin_name,
+  const ::sql::SQLString & option,
+  const ::sql::SQLString & value,
+  bool default_value
+)
 {
-  std::string err(e.what());
-  err+= " for plugin " + plugin_name;
-  throw sql::InvalidArgumentException(err);
+  return plugin_option(
+    plugin_type, plugin_name, option, value.c_str(), default_value
+  );
 }
 
+
 int MySQL_NativeConnectionWrapper::get_plugin_option(
     int plugin_type,
     const ::sql::SQLString & plugin_name,
     const ::sql::SQLString & option,
     const ::sql::SQLString & value)
 {
-
+  PluginGuard guard{this};
 
   /* load client authentication plugin if required */
-  struct st_mysql_client_plugin *plugin =
-      api->client_find_plugin(mysql, plugin_name.c_str(),
-                              plugin_type);
+  struct st_mysql_client_plugin *plugin = guard.get_plugin(plugin_type, plugin_name, true);
 
   /* get option value from plugin */
   return api->plugin_get_option(plugin, option.c_str(), (void*)value.c_str());
@@ -506,7 +663,25 @@ MySQL_NativeConnectionWrapper::query(const SQLString & stmt_str)
 int
 MySQL_NativeConnectionWrapper::ping()
 {
-  return api->ping(mysql);
+  int res = api->ping(mysql);
+
+#if MYCPPCONN_STATIC_MYSQL_VERSION_ID >= 80400
+  if (res && reconnect) {
+    // Try reconnecting if could not ping and reconnect
+    // option is set.
+    bool connect_result = !m_dns_srv ?
+            connect(m_host, m_user, m_passwd, m_db, m_port,
+                    m_socket_or_pipe, m_client_flag)
+            :
+            connect_dns_srv(m_host, m_user, m_passwd, m_db,
+                    m_client_flag);
+    // If connected return success, otherwise let ping()
+    // return the proper error.
+    res = connect_result ? 0 : api->ping(mysql);
+  }
+#endif
+
+  return res;
 }
 /* }}} */
 
@@ -539,20 +714,6 @@ MySQL_NativeConnectionWrapper::info()
 /* }}} */
 
 
-/* {{{ MySQL_NativeConnectionWrapper::ssl_set() */
-bool
-MySQL_NativeConnectionWrapper::ssl_set(const SQLString & key,
-                const SQLString & cert,
-                const SQLString & ca,
-                const SQLString & capath,
-                const SQLString & cipher)
-{
-  return ('\0' != api->ssl_set(mysql, nullIfEmpty(key), nullIfEmpty(cert),
-              nullIfEmpty(ca), nullIfEmpty(capath), nullIfEmpty(cipher)));
-}
-/* }}} */
-
-
 /* {{{ MySQL_NativeConnectionWrapper::store_result() */
 NativeResultsetWrapper *
 MySQL_NativeConnectionWrapper::store_result()
