@@ -1,22 +1,22 @@
 /*
- * Copyright (c) 2009, 2020, Oracle and/or its affiliates.
+ * Copyright (c) 2009, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
  * published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
- * documentation.  The authors of MySQL hereby grant you an
- * additional permission to link the program and your derivative works
- * with the separately licensed software that they have included with
- * MySQL.
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation. The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * Without limiting anything contained in the foregoing, this file,
- * which is part of MySQL Connector/C++, is also subject to the
+ * which is part of Connector/C++, is also subject to the
  * Universal FOSS Exception, version 1.0, a copy of which can be found at
- * http://oss.oracle.com/licenses/universal-foss-exception.
+ * https://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +25,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
@@ -820,7 +820,6 @@ void bugs::bug21066575()
 
 void bugs::bug14520822()
 {
-
   logMsg("bug::bug14520822");
   try
   {
@@ -1271,6 +1270,39 @@ void bugs::bug23235968()
 
 }
 
+void bugs::bug19192707()
+{
+  logMsg("bugs::bug19192707");
+
+  sql::SQLString direct, ps;
+
+  stmt->executeUpdate("drop table if exists bug19192707");
+  stmt->executeUpdate("create table bug19192707 (i1 point)");
+
+  pstmt.reset(con->prepareStatement("insert into bug19192707 (i1) values(ST_PointFromText('POINT(1 2)',4326))"));
+  pstmt->executeUpdate();
+
+  logMsg("Select with Direct execute : ");
+
+  res.reset(stmt->executeQuery("select i1 from bug19192707"));
+
+  res->next();
+  direct = res->getString(1);
+
+  logMsg("Select with Prepare/execute : ");
+
+  pstmt.reset(con->prepareStatement("select i1 from bug19192707"));
+
+  res.reset(pstmt->executeQuery());
+
+  res->next();
+  ps = res->getString(1);
+
+  ASSERT_EQUALS(direct, ps);
+
+  stmt->executeUpdate("drop table if exists bug19192707");
+
+}
 
 } /* namespace regression */
 } /* namespace testsuite */
