@@ -1,22 +1,22 @@
 /*
- * Copyright (c) 2008, 2020, Oracle and/or its affiliates.
+ * Copyright (c) 2008, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
  * published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
- * documentation.  The authors of MySQL hereby grant you an
- * additional permission to link the program and your derivative works
- * with the separately licensed software that they have included with
- * MySQL.
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation. The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * Without limiting anything contained in the foregoing, this file,
- * which is part of MySQL Connector/C++, is also subject to the
+ * which is part of Connector/C++, is also subject to the
  * Universal FOSS Exception, version 1.0, a copy of which can be found at
- * http://oss.oracle.com/licenses/universal-foss-exception.
+ * https://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +25,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
@@ -537,38 +537,19 @@ void connectionmetadata::getColumns()
     }
 
     try {
-      bool input_value=true;
-      bool output_value=false;
-      void * input;
-      void * output;
-
       stmt->execute("CREATE TABLE test(id INT,val VARCHAR(20))");
 
-      input=(static_cast<bool *> (&input_value));
-      output=(static_cast<bool *> (&output_value));
-
-      con->setClientOption("metadataUseInfoSchema", input);
-      con->getClientOption("metadataUseInfoSchema", output);
-      ASSERT_EQUALS(input_value, output_value);
-
       dbmeta=con->getMetaData();
       res.reset(dbmeta->getColumns(con->getCatalog(), "", "test", "%"));
       ASSERT(res->rowsCount() == 2);
 
-      input_value=false;
-      output_value=true;
-
-      con->setClientOption("metadataUseInfoSchema", input);
-      con->getClientOption("metadataUseInfoSchema", output);
-      ASSERT_EQUALS(input_value, output_value);
-
       dbmeta=con->getMetaData();
       res.reset(dbmeta->getColumns(con->getCatalog(), "", "test", "%"));
       ASSERT(res->rowsCount() == 2);
     }
     catch (sql::SQLException &)
     {
-      FAIL("getColumns() does not work properly for metadataUseInfoSchema");
+      FAIL("getColumns() does not work properly");
     }
 
     stmt->execute("DROP TABLE IF EXISTS test");
@@ -617,7 +598,7 @@ void connectionmetadata::getDatabaseVersions()
   {
     DatabaseMetaData * dbmeta=con->getMetaData();
     ASSERT_GT(2, dbmeta->getDatabaseMajorVersion());
-    ASSERT_LT(8, dbmeta->getDatabaseMajorVersion());
+    ASSERT_LT(9, dbmeta->getDatabaseMajorVersion());
     ASSERT_LT(100, dbmeta->getDatabaseMinorVersion());
     ASSERT_LT(100, dbmeta->getDatabasePatchVersion());
 
@@ -653,7 +634,7 @@ void connectionmetadata::getDriverVersions()
   {
     DatabaseMetaData * dbmeta=con->getMetaData();
     ASSERT_GT(0, dbmeta->getDriverMajorVersion());
-    ASSERT_LT(8, dbmeta->getDriverMajorVersion());
+    ASSERT_LT(9, dbmeta->getDriverMajorVersion());
     ASSERT_LT(100, dbmeta->getDriverMinorVersion());
     ASSERT_LT(100, dbmeta->getDriverPatchVersion());
 
@@ -1487,117 +1468,96 @@ void connectionmetadata::getProcedures()
 
   sql::ConnectOptionsMap options;
 
-  for(int i=0; i < 2; ++i)
+  con.reset(unit_fixture::getConnection(&options));
+  con->setSchema(db);
+  bool got_warning=false;
+  std::stringstream msg;
+  try
   {
-    options[OPT_METADATA_INFO_SCHEMA] = i == 0;
-
-    con.reset(unit_fixture::getConnection(&options));
-    con->setSchema(db);
-
-
-    bool got_warning=false;
-    std::stringstream msg;
+    DatabaseMetaData * dbmeta=con->getMetaData();
+    stmt.reset(con->createStatement());
     try
     {
-      DatabaseMetaData * dbmeta=con->getMetaData();
-      stmt.reset(con->createStatement());
-      try
-      {
-        stmt->execute("DROP PROCEDURE IF EXISTS p1");
-        stmt->execute("CREATE PROCEDURE p1(OUT param1 INT) COMMENT 'Sample Procedure' BEGIN SELECT 1 INTO param1; END");
-        stmt->execute("DROP FUNCTION IF EXISTS f1");
-        stmt->execute("CREATE FUNCTION f1(s CHAR(20)) RETURNS CHAR(50) COMMENT 'Sample Function'  DETERMINISTIC RETURN CONCAT('Hello, ',s,'!')");
-      }
-      catch (sql::SQLException &)
-      {
-        SKIP("Cannot create procedure");
-      }
-
-      // Verify if the procedure creally has been created...
-      stmt->execute("SET @myvar = -1");
-      stmt->execute("CALL p1(@myvar)");
-
-      res.reset(stmt->executeQuery("SELECT @myvar AS _myvar"));
-      ASSERT(res->next());
-      ASSERT_EQUALS(1, res->getInt("_myvar"));
-      logMsg("...who is the bad guy?");
-
-      res.reset(dbmeta->getProcedures(con->getCatalog(), con->getSchema(), "p1"));
-      checkResultSetScrolling(res);
-      logMsg("...is it you, getProcedures()?");
-      ASSERT(res->next());
-
-      if (con->getCatalog() != "" && res->getString("PROCEDURE_CAT") != "" && con->getCatalog() != res->getString("PROCEDURE_CAT"))
-      {
-        got_warning=true;
-        msg.str("");
-        msg << "\t\tWARNING expecting PROCEDURE_CAT = '" << con->getCatalog() << "'";
-        msg << " got '" << res->getString("PROCEDURE_CAT") << "'";
-        logMsg(msg.str());
-      }
-
-      ASSERT_EQUALS(res->getString(1), res->getString("PROCEDURE_CAT"));
-      ASSERT_EQUALS(con->getSchema(), res->getString("PROCEDURE_SCHEM"));
-      ASSERT_EQUALS(res->getString(2), res->getString("PROCEDURE_SCHEM"));
-      ASSERT_EQUALS("p1", res->getString(3));
-      ASSERT_EQUALS(res->getString(3), res->getString("PROCEDURE_NAME"));
-      ASSERT_EQUALS("", res->getString(4));
-      ASSERT_EQUALS("", res->getString(5));
-      ASSERT_EQUALS("", res->getString(6));
-      ASSERT_EQUALS("Sample Procedure", res->getString(7));
-      ASSERT_EQUALS(res->getString("REMARKS"), res->getString(7));
-      ASSERT_EQUALS(DatabaseMetaData::procedureNoResult, res->getInt("PROCEDURE_TYPE"));
-      ASSERT(DatabaseMetaData::procedureReturnsResult != res->getInt(8));
-      ASSERT(DatabaseMetaData::procedureResultUnknown != res->getInt(8));
-      ASSERT(!res->next());
-
-      res.reset(dbmeta->getProcedures(con->getCatalog(), con->getSchema(), "f1"));
-      checkResultSetScrolling(res);
-
-      ASSERT(res->next());
-
-      if (con->getCatalog() != "" && res->getString("PROCEDURE_CAT") != "" && con->getCatalog() != res->getString("PROCEDURE_CAT"))
-      {
-        got_warning=true;
-        msg.str("");
-        msg << "\t\tWARNING expecting PROCEDURE_CAT = '" << con->getCatalog() << "'";
-        msg << " got '" << res->getString("PROCEDURE_CAT") << "'";
-        logMsg(msg.str());
-      }
-
-      ASSERT_EQUALS(res->getString(1), res->getString("PROCEDURE_CAT"));
-      ASSERT_EQUALS(con->getSchema(), res->getString("PROCEDURE_SCHEM"));
-      ASSERT_EQUALS(res->getString(2), res->getString("PROCEDURE_SCHEM"));
-      ASSERT_EQUALS("f1", res->getString(3));
-      ASSERT_EQUALS(res->getString(3), res->getString("PROCEDURE_NAME"));
-      ASSERT_EQUALS("", res->getString(4));
-      ASSERT_EQUALS("", res->getString(5));
-      ASSERT_EQUALS("", res->getString(6));
-      ASSERT_EQUALS("Sample Function", res->getString(7));
-      ASSERT_EQUALS(res->getString("REMARKS"), res->getString(7));
-      ASSERT_EQUALS(DatabaseMetaData::procedureReturnsResult, res->getInt("PROCEDURE_TYPE"));
-      ASSERT(DatabaseMetaData::procedureNoResult != res->getInt(8));
-      ASSERT(DatabaseMetaData::procedureResultUnknown != res->getInt(8));
-      ASSERT(!res->next());
+      stmt->execute("DROP PROCEDURE IF EXISTS p1");
+      stmt->execute("CREATE PROCEDURE p1(OUT param1 INT) COMMENT 'Sample Procedure' BEGIN SELECT 1 INTO param1; END");
+      stmt->execute("DROP FUNCTION IF EXISTS f1");
+      stmt->execute("CREATE FUNCTION f1(s CHAR(20)) RETURNS CHAR(50) COMMENT 'Sample Function'  DETERMINISTIC RETURN CONCAT('Hello, ',s,'!')");
     }
-    catch (sql::SQLException &e)
+    catch (sql::SQLException &)
     {
-      logErr(e.what());
-      logErr("SQLState: " + std::string(e.getSQLState()));
-      fail(e.what(), __FILE__, __LINE__);
+      SKIP("Cannot create procedure");
     }
-
-    if (got_warning)
+    // Verify if the procedure creally has been created...
+    stmt->execute("SET @myvar = -1");
+    stmt->execute("CALL p1(@myvar)");
+    res.reset(stmt->executeQuery("SELECT @myvar AS _myvar"));
+    ASSERT(res->next());
+    ASSERT_EQUALS(1, res->getInt("_myvar"));
+    logMsg("...who is the bad guy?");
+    res.reset(dbmeta->getProcedures(con->getCatalog(), con->getSchema(), "p1"));
+    checkResultSetScrolling(res);
+    logMsg("...is it you, getProcedures()?");
+    ASSERT(res->next());
+    if (con->getCatalog() != "" && res->getString("PROCEDURE_CAT") != "" && con->getCatalog() != res->getString("PROCEDURE_CAT"))
     {
-      if(i==0){
-        TODO("Using Information Schema")
-      } else {
-        TODO("Not Using Information Schema");
-      }
-      TODO("See --verbose warnings!");
-      FAIL("TODO - see --verbose warnings!");
+      got_warning=true;
+      msg.str("");
+      msg << "\t\tWARNING expecting PROCEDURE_CAT = '" << con->getCatalog() << "'";
+      msg << " got '" << res->getString("PROCEDURE_CAT") << "'";
+      logMsg(msg.str());
     }
+    ASSERT_EQUALS(res->getString(1), res->getString("PROCEDURE_CAT"));
+    ASSERT_EQUALS(con->getSchema(), res->getString("PROCEDURE_SCHEM"));
+    ASSERT_EQUALS(res->getString(2), res->getString("PROCEDURE_SCHEM"));
+    ASSERT_EQUALS("p1", res->getString(3));
+    ASSERT_EQUALS(res->getString(3), res->getString("PROCEDURE_NAME"));
+    ASSERT_EQUALS("", res->getString(4));
+    ASSERT_EQUALS("", res->getString(5));
+    ASSERT_EQUALS("", res->getString(6));
+    ASSERT_EQUALS("Sample Procedure", res->getString(7));
+    ASSERT_EQUALS(res->getString("REMARKS"), res->getString(7));
+    ASSERT_EQUALS(DatabaseMetaData::procedureNoResult, res->getInt("PROCEDURE_TYPE"));
+    ASSERT(DatabaseMetaData::procedureReturnsResult != res->getInt(8));
+    ASSERT(DatabaseMetaData::procedureResultUnknown != res->getInt(8));
+    ASSERT(!res->next());
+    res.reset(dbmeta->getProcedures(con->getCatalog(), con->getSchema(), "f1"));
+    checkResultSetScrolling(res);
+    ASSERT(res->next());
+    if (con->getCatalog() != "" && res->getString("PROCEDURE_CAT") != "" && con->getCatalog() != res->getString("PROCEDURE_CAT"))
+    {
+      got_warning=true;
+      msg.str("");
+      msg << "\t\tWARNING expecting PROCEDURE_CAT = '" << con->getCatalog() << "'";
+      msg << " got '" << res->getString("PROCEDURE_CAT") << "'";
+      logMsg(msg.str());
+    }
+    ASSERT_EQUALS(res->getString(1), res->getString("PROCEDURE_CAT"));
+    ASSERT_EQUALS(con->getSchema(), res->getString("PROCEDURE_SCHEM"));
+    ASSERT_EQUALS(res->getString(2), res->getString("PROCEDURE_SCHEM"));
+    ASSERT_EQUALS("f1", res->getString(3));
+    ASSERT_EQUALS(res->getString(3), res->getString("PROCEDURE_NAME"));
+    ASSERT_EQUALS("", res->getString(4));
+    ASSERT_EQUALS("", res->getString(5));
+    ASSERT_EQUALS("", res->getString(6));
+    ASSERT_EQUALS("Sample Function", res->getString(7));
+    ASSERT_EQUALS(res->getString("REMARKS"), res->getString(7));
+    ASSERT_EQUALS(DatabaseMetaData::procedureReturnsResult, res->getInt("PROCEDURE_TYPE"));
+    ASSERT(DatabaseMetaData::procedureNoResult != res->getInt(8));
+    ASSERT(DatabaseMetaData::procedureResultUnknown != res->getInt(8));
+    ASSERT(!res->next());
   }
+  catch (sql::SQLException &e)
+  {
+    logErr(e.what());
+    logErr("SQLState: " + std::string(e.getSQLState()));
+    fail(e.what(), __FILE__, __LINE__);
+  }
+  if (got_warning)
+  {
+    TODO("See --verbose warnings!");
+    FAIL("TODO - see --verbose warnings!");
+  }
+
   con.reset(unit_fixture::getConnection());
   con->setSchema(db);
 }
@@ -2111,13 +2071,12 @@ void connectionmetadata::getColumnsTypeConversions()
   std::vector<columndefinition>::iterator it;
   std::stringstream msg;
   int i;
-  bool got_warning;
+  bool got_warning=false;
   try
   {
     DatabaseMetaData * dbmeta=con->getMetaData();
     stmt.reset(con->createStatement());
 
-    got_warning=false;
     logMsg("... looping over all kinds of column types");
     for (it=columns.begin(), i=0; it != columns.end(); i++, it++)
     {
@@ -2340,7 +2299,7 @@ void connectionmetadata::getSchemaCollation()
   ASSERT_EQUALS("SCHEMA_COLLATION", resmeta->getColumnLabel(3));
 
   ASSERT(res->getString("SCHEMA_NAME").caseCompare("collationTestDatabase") == 0);
-  ASSERT_EQUALS("utf8_bin", res->getString("SCHEMA_COLLATION"));
+  ASSERT_EQUALS("utf8mb3_bin", res->getString("SCHEMA_COLLATION"));
 
   stmt->execute("DROP DATABASE IF EXISTS collationTestDatabase");
   }
@@ -2423,7 +2382,7 @@ void connectionmetadata::getTableCollation()
   ASSERT(res->next());
   ASSERT(res->getString("TABLE_SCHEMA").caseCompare("collationTestDatabase") == 0);
   ASSERT(res->getString("TABLE_NAME").caseCompare("collationTestTableAnother") == 0);
-  ASSERT_EQUALS("utf8_bin", res->getString("TABLE_COLLATION"));
+  ASSERT_EQUALS("utf8mb3_bin", res->getString("TABLE_COLLATION"));
 
   stmt->execute("DROP DATABASE IF EXISTS collationTestDatabase");
   }
