@@ -1,22 +1,22 @@
 /*
- * Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
  * published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
- * documentation.  The authors of MySQL hereby grant you an
- * additional permission to link the program and your derivative works
- * with the separately licensed software that they have included with
- * MySQL.
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation. The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * Without limiting anything contained in the foregoing, this file,
- * which is part of MySQL Connector/C++, is also subject to the
+ * which is part of Connector/C++, is also subject to the
  * Universal FOSS Exception, version 1.0, a copy of which can be found at
- * http://oss.oracle.com/licenses/universal-foss-exception.
+ * https://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,7 +25,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
@@ -200,9 +200,6 @@ void statement::callSP()
     connection_properties["userName"]=user;
     connection_properties["password"]=passwd;
 
-    bool bval= !TestsRunner::getStartOptions()->getBool("dont-use-is");
-    connection_properties["metadataUseInfoSchema"]=bval;
-
     connection_properties.erase("CLIENT_MULTI_RESULTS");
     connection_properties["CLIENT_MULTI_RESULTS"]=true;
 
@@ -344,9 +341,6 @@ void statement::unbufferedFetch()
     connection_properties["userName"]=user;
     connection_properties["password"]=passwd;
 
-    bool bval= !TestsRunner::getStartOptions()->getBool("dont-use-is");
-    connection_properties["metadataUseInfoSchema"]=bval;
-
     logMsg("... setting TYPE_FORWARD_ONLY through connection map");
     connection_properties.erase("defaultStatementResultType");
     {
@@ -692,6 +686,28 @@ void statement::queryAttributes()
   ASSERT(res->isNull(3));
   ASSERT(res->isNull(4));
 
+  // Scenario 5 - traceparent behavior
+  stmt->clearAttributes();
+  res.reset(stmt->executeQuery("SELECT 1,"
+                               "mysql_query_attribute_string('traceparent')"));
+
+  ASSERT(res->next());
+  ASSERT_EQUALS(1, res->getInt(1));
+  if (!res->isNull(2))
+  {
+    // At this point we know that traceparent was set by telemetry.
+    // Set user traceparent attribute and see that it overwrites the one set
+    // by telemetry.
+    sql::SQLString s = "user set string";
+
+    stmt->setQueryAttrString("traceparent", s);
+    res.reset(stmt->executeQuery("SELECT 2,"
+                                "mysql_query_attribute_string('traceparent')"));
+    ASSERT(res->next());
+    ASSERT_EQUALS(2, res->getInt(1));
+    ASSERT_EQUALS(s, res->getString(2));
+  }
+
   stmt->execute("UNINSTALL COMPONENT 'file://component_query_attributes'");
 
 }
