@@ -1,22 +1,22 @@
 /*
- * Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
  * published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
- * documentation.  The authors of MySQL hereby grant you an
- * additional permission to link the program and your derivative works
- * with the separately licensed software that they have included with
- * MySQL.
+ * This program is designed to work with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation. The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * Without limiting anything contained in the foregoing, this file,
- * which is part of MySQL Connector/C++, is also subject to the
+ * which is part of Connector/C++, is also subject to the
  * Universal FOSS Exception, version 1.0, a copy of which can be found at
- * http://oss.oracle.com/licenses/universal-foss-exception.
+ * https://oss.oracle.com/licenses/universal-foss-exception.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,11 +25,11 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#include <iostream>
 #include <mysqlx/xdevapi.h>
+#include <iostream>
 
 
 using ::std::cout;
@@ -48,10 +48,6 @@ try {
   Session sess(url);
 
   {
-  /*
-    TODO: Only working with server version 8
-  */
-
     RowResult res = sess.sql("show variables like 'version'").execute();
     std::stringstream version;
 
@@ -61,18 +57,19 @@ try {
 
     if (major_version < 8)
     {
-      cout <<"Done!" <<endl;
+      cout << "Can work only with MySQL Server 8 or later" << endl;
+      cout << "Done!" << endl;
       return 0;
     }
   }
 
 
-  cout <<"Session accepted, creating collection..." <<endl;
+  cout << "Session accepted, creating collection..." << endl;
 
   Schema sch = sess.createSchema("test", true);
   Collection coll= sch.createCollection("c1", true);
 
-  cout <<"Inserting documents..." <<endl;
+  cout << "Inserting documents..." << endl;
 
   coll.remove("true").execute();
 
@@ -80,48 +77,53 @@ try {
     Result add;
 
     add= coll.add(R"({ "name": "foo", "age": 1 })").execute();
+
     std::vector<string> ids = add.getGeneratedIds();
-    cout <<"- added doc with id: " << ids[0] <<endl;
+    cout << "- added doc with id: " << ids[0] << endl;
 
     add= coll.add(R"({ "name": "bar", "age": 2, "toys": [ "car", "ball" ] })")
              .execute();
+
+    ids = add.getGeneratedIds();
     if (ids.size() != 0)
-      cout <<"- added doc with id: " << ids[0] <<endl;
+      cout << "- added doc with id: " << ids[0] << endl;
     else
-      cout <<"- added doc" <<endl;
+      cout << "- added doc" << endl;
 
     add= coll.add(R"({
        "name": "baz",
         "age": 3,
        "date": { "day": 20, "month": "Apr" }
     })").execute();
+
+    ids = add.getGeneratedIds();
     if (ids.size() != 0)
-      cout <<"- added doc with id: " << ids[0] <<endl;
+      cout << "- added doc with id: " << ids[0] << endl;
     else
-      cout <<"- added doc" <<endl;
+      cout << "- added doc" << endl;
 
     add= coll.add(R"({ "_id": "myuuid-1", "name": "foo", "age": 7 })")
              .execute();
+
     ids = add.getGeneratedIds();
     if (ids.size() != 0)
-      cout <<"- added doc with id: " << ids[0] <<endl;
+      cout << "- added doc with id: " << ids[0] << endl;
     else
-      cout <<"- added doc" <<endl;
+      cout << "- added doc" << endl;
   }
 
-  cout <<"Fetching documents..." <<endl;
+  cout << "Fetching documents..." << endl;
 
   DocResult docs = coll.find("age > 1 and name like 'ba%'").execute();
 
-  DbDoc doc = docs.fetchOne();
-
-  for (int i = 0; doc; ++i, doc = docs.fetchOne())
+  int i = 0;
+  for (DbDoc doc : docs)
   {
-    cout <<"doc#" <<i <<": " <<doc <<endl;
+    cout << "doc#" << i++ << ": " << doc << endl;
 
     for (Field fld : doc)
     {
-      cout << " field `" << fld << "`: " <<doc[fld] << endl;
+      cout << " field `" << fld << "`: " << doc[fld] << endl;
     }
 
     string name = doc["name"];
@@ -153,20 +155,20 @@ try {
     cout << endl;
   }
 
-  cout <<"Done!" <<endl;
+  cout << "Done!" << endl;
 }
 catch (const mysqlx::Error &err)
 {
-  cout <<"ERROR: " <<err <<endl;
+  cout << "ERROR: " << err << endl;
   return 1;
 }
 catch (std::exception &ex)
 {
-  cout <<"STD EXCEPTION: " <<ex.what() <<endl;
+  cout << "STD EXCEPTION: " << ex.what() << endl;
   return 1;
 }
 catch (const char *ex)
 {
-  cout <<"EXCEPTION: " <<ex <<endl;
+  cout << "EXCEPTION: " << ex << endl;
   return 1;
 }
